-- Andrew McCallum's cheese maze POMDP

-- program options
debug_mode = false;

-- initialise prng
seed = os.time()
math.randomseed(seed)
math.random(); math.random(); math.random();

-- encoding/decoding tables
encode_obs = {}
encode_obs[1]  = '0111'
encode_obs[2]  = '0101'
encode_obs[3]  = '1001'
encode_obs[4]  = '1010'
encode_obs[5]  = '0111'
encode_obs[6]  = '0101'
encode_obs[7]  = '1000'
encode_obs[8]  = '1010'
encode_obs[9]  = '0111'
encode_obs[10] = '0101'
encode_obs[11] = '1100'


encode_rew = {}
encode_rew[0]   = '00000'
encode_rew[9]   = '01001'
encode_rew[20]  = '10100'

decode_act = {}
decode_act['00'] = 'north'
decode_act['01'] = 'east'
decode_act['10'] = 'south'
decode_act['11'] = 'west'


-- encode the maze connectivity structure
maze = {}
maze[1]  = {}; maze[1]['north']  = 2
maze[2]  = {}; maze[2]['south']  = 1;  maze[2]['north'] = 3
maze[3]  = {}; maze[3]['south']  = 2;  maze[3]['east']  = 4
maze[4]  = {}; maze[4]['west']   = 3;  maze[4]['east']  = 7
maze[5]  = {}; maze[5]['north']  = 6
maze[6]  = {}; maze[6]['south']  = 5;  maze[6]['north'] = 7
maze[7]  = {}; maze[7]['west']   = 4;  maze[7]['south'] = 6;  maze[7]['east'] = 8
maze[8]  = {}; maze[8]['west']   = 7;  maze[8]['east']  = 11
maze[9]  = {}; maze[9]['north']  = 10
maze[10] = {}; maze[10]['south'] = 9;  maze[10]['north'] = 11
maze[11] = {}; maze[11]['west']  = 8;  maze[11]['south'] = 10



-- reset the POMDP to the starting state
function assign_cheese_and_mouse ()

	cheese_state = 5
	mouse_state  = math.random(11)
	bump_state   = false

	dprint('cheese_state =' .. cheese_state)
	dprint('mouse_state  =' .. mouse_state)
end


-- compute the reward for an agent based on the world state
function compute_reward ()

	if (cheese_state == mouse_state) then
		return 20
	elseif (bump_state) then
		return 0
	else
		return 9
	end

end


-- compute the next observation based on the world state
function compute_observation ()

	return encode_obs[mouse_state]

end


-- determine the next state
function compute_next_state (action)

	-- if we have found the cheese, reset
	if (mouse_state == cheese_state) then
		assign_cheese_and_mouse()
		return
	end

	bump_state = false;

	-- if mouse bumped into wall
	if (maze[mouse_state][action] == nil) then
		bump_state = true
	else --mouse moved into free cell
		mouse_state = maze[mouse_state][action]
	end

end


-- debugging print
function dprint (str)

	if (debug_mode) then
		local f = io.output()
		io.output(stderr)
		print(str)
		io.output(f)
	end
end



-- main loop
line   = ''
assign_cheese_and_mouse()

repeat

	local obs = compute_observation()
	local rew = compute_reward()

	dprint('obs=' .. obs .. ', rew=' .. rew)

	local enc_rew = encode_rew[rew]

	print(obs .. enc_rew)
	io.flush()


	-- grab action
	line = io.read()
	if (line == '') then break end
	action = decode_act[line];
	dprint('action=' .. action)

	compute_next_state(action)


until line == ''




