
-- scissor paper rock versus an opponent who plays randomly
-- unless they just won by playing

-- program options
debug_mode = false;

-- initialise prng
seed = os.time()
math.randomseed(seed)
math.random(); math.random(); math.random();

-- encoding/decoding tables
encode_obs = {}
encode_obs.null     = '00'
encode_obs.rock     = '01'
encode_obs.scissors = '10'
encode_obs.paper    = '11'

encode_rew = {}
encode_rew[0]   = '00'
encode_rew[1]   = '01'
encode_rew[2]   = '10'

decode_act = {}
decode_act['00'] = 'rock'
decode_act['01'] = 'scissors'
decode_act['10'] = 'paper'


last_us_action  = nil
last_opp_action = 'null'
opp_action      = nil


-- compute the reward for an agent given an action
function compute_reward (action)

	-- if we picked scissors
	if (action == 'scissors') then
		if (opp_action == 'rock') then
			return 0
		elseif (opp_action == 'paper') then
			return 2
		else
			return 1
		end
	end

	-- if we picked rock
	if (action == 'rock') then
		if (opp_action == 'paper') then
			return 0
		elseif (opp_action == 'scissors') then
			return 2
		else
			return 1
		end
	end

	-- if we picked paper
	if (action == 'paper') then
		if (opp_action == 'scissors') then
			return 0
		elseif (opp_action == 'rock') then
			return 2
		else
			return 1
		end
	end

end


-- compute the next observation for an agent given an action
function compute_observation ()
	return encode_obs[last_opp_action]
end


-- get the opponent action
function get_opponent_action ()

	if (last_us_action == 'scissors' and last_opp_action == 'rock') then
		return 'rock'
	end

	local r = math.random()

	if (r < 0.33333333) then
		return 'paper'
	elseif (r < 0.666666666) then
		return 'scissors'
	else
		return 'rock'
	end

end


-- debugging print
function dprint (str)

	if (debug_mode) then
		local f = io.output()
		io.output(stderr)
		print(str)
		io.output(f)
	end
end


-- main loop: play the tiger POMDP
--            until a blank line is seen

line   = ''

-- display dummy observation/reward
print(encode_obs.null .. encode_rew[0])
io.flush()

repeat

	opp_action = get_opponent_action()
	dprint('opp_action=' .. opp_action)

	-- grab action
	line = io.read()
	if (line == '') then break end

	action = decode_act[line];
	dprint('action=' .. action)
	local obs = compute_observation(action)
	local rew = compute_reward(action)

	dprint('obs=' .. obs .. ', rew=' .. rew)

	local enc_rew = encode_rew[rew]

	print(obs .. enc_rew)
	io.flush()

	last_us_action  = action
	last_opp_action = opp_action

until line == ''




