
-- program options
debug_mode = false;

-- initialise prng
seed = os.time()
math.randomseed(seed)
math.random(); math.random(); math.random();

-- encoding/decoding tables
encode_obs = {}
encode_obs.null        = '000'
encode_obs.sit_left    = '010'
encode_obs.sit_right   = '011'
encode_obs.stand_left  = '110'
encode_obs.stand_right = '111'

encode_rew = {}
encode_rew.null = '01100100'
encode_rew[0]   = '00000000'
encode_rew[99]  = '01100011'
encode_rew[100] = '01100100'
encode_rew[130] = '10000010'

decode_act = {}
decode_act['00'] = 'listen'
decode_act['01'] = 'left'
decode_act['10'] = 'right'
decode_act['11'] = 'stand'


-- reset the POMDP to the starting state
function assign_tiger ()

	sit_state   = 'sitting'

	local r = math.random()
	if (r < 0.5) then
		tiger_state = 'left'
		safe_state  = 'right'
	else
		tiger_state = 'right'
		safe_state  = 'left'
	end
end


-- compute the reward for an agent given an action
function compute_reward (action)

	-- we can't open the door succesfully if we are
	-- sitting, we just waste time
	if (sit_state == 'sitting') then
		if (action == 'left' or action == 'right') then
			-- illegal action
			return 0
		end
	end

	if (sit_state == 'standing') then
		if (action == 'stand') then
			-- illegal action
			return 0
		end
	end

	if (action == tiger_state) then
		-- we get eaten
		return 0
	elseif (action == 'stand') then
		return 99
	elseif (action == 'listen') then
		if (sit_state == 'sitting') then
			return 100
		else
			return 99
		end
	else -- we open the right door
		return 130
	end

end


-- compute the next observation for an agent given an action
function compute_observation (action)

	if (action == 'listen') then
		if (sit_state == 'sitting') then
			if (math.random() < 0.85) then
				return 'sit_' .. tiger_state
			else
				return 'sit_' .. safe_state
			end
		else
			if (math.random() < 0.50) then
				return 'stand_left'
			else
				return 'stand_right'
			end
		end
	end

	return 'null'
end


-- debugging print
function dprint (str)

	if (debug_mode) then
		local f = io.output()
		io.output(stderr)
		print(str)
		io.output(f)
	end
end


-- main loop: play the tiger POMDP
--            until a blank line is seen

action = 'left'
line   = ''
rew = 0

-- display dummy observation/reward
print(encode_obs.null .. encode_rew[130])
io.flush()

repeat

	if (action == 'left' or action == 'right' or rew == 0) then
		-- reset game
		assign_tiger()
		dprint('tiger=' .. tiger_state)
	end

	-- grab action
	line = io.read()
	if (line == '') then break end

	action = decode_act[line];

	dprint('action=' .. action)
	local obs = compute_observation(action)
	rew = compute_reward(action)

	dprint('obs=' .. obs .. ', rew=' .. rew)

	local enc_obs = encode_obs[obs]
	local enc_rew = encode_rew[rew]

	print(enc_obs .. enc_rew)
	io.flush()

	-- modify state if we just stood up
	if (action == 'stand') then
		sit_state = 'standing'
	end


until line == ''




